package eu.ztsh.wymiana.service;

import eu.ztsh.wymiana.EntityCreator;
import eu.ztsh.wymiana.RepositoryBasedTest;
import eu.ztsh.wymiana.data.repository.UserRepository;
import eu.ztsh.wymiana.util.UserMapper;
import eu.ztsh.wymiana.web.model.UserCreateRequest;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

class UserServiceTest extends RepositoryBasedTest {

    private final UserService userService = new UserService();

    @Autowired
    public UserServiceTest(UserRepository userRepository) {
        super(userRepository);
    }

    @Test
    void createNewUserTest() {
        userService.create(createRequest());
        var entity = EntityCreator.user().build();
        expect(entity);
    }

    @Test
    void createDuplicatedUser() {
        var first = createRequest();
        var second = createRequest();
        userService.create(first);
        assertThatThrownBy(() -> userService.create(second))
            .isInstanceOf(IllegalArgumentException.class)
            .hasMessage("Duplicated PESEL: %s".formatted(EntityCreator.Constants.PESEL));
    }

    @Test
    void getExistingUserTest() {
        var entity = EntityCreator.user().build();
        userRepository.save(entity);
        var userOptional = userService.get(EntityCreator.Constants.PESEL);
        var expected = UserMapper.entityToPojo(entity);
        assertThat(userOptional)
            .isNotEmpty()
            .get()
            .usingRecursiveComparison()
            .isEqualTo(expected);
    }

    @Test
    void getNonExistingUserTest() {
        var userOptional = userService.get(EntityCreator.Constants.PESEL);
        assertThat(userOptional).isEmpty();
    }

    private UserCreateRequest createRequest() {
        return new UserCreateRequest(EntityCreator.Constants.NAME, EntityCreator.Constants.SURNAME,
            EntityCreator.Constants.PESEL, 20.10);
    }

}
